/* Sistemas Integrados TEKNISA
   Controle de Produo
Nome: CDPFUNO_POSESTCONS_LOTE_VALIDO.SQL

Descrio:
Retorna a posio de estoque consolidada de um produto em relao a um almoxarifado cujo lote esteja valido, quando comparado a data de referencia
passada por parametro(P_DTREFVALIDADE).

P_FILIAL        - Cdigo da Filial.
P_PRODUTO       - Cdigo do Produto.
P_ALMOXARIFE    - Codigo do Almoxarifado (se vazio'', indica que o sistema ira buscar o almox. com base em parametrizaes),
P_CDLOCALESTOQ  - Codigo do Local de Estoque(se vazio '', indica que o sistema ira buscar o almox. com base em parametrizaes)
P_IDPRODACABADO - Identifica se  um produto acabado (S ou N).
P_DTPOSICAO     - Data da posio.
P_DTREFVALIDADE - Data de referencia para a validade.Para ser considerado valido, um lote precisa ser maior ou igual a data passada
neste parmetro. 


@C:\Web_649\Sql\End_Conv\Oracle\4431_CDPFUNO_POSESTCONS_LOTE_VALIDO.SQL

Criao:
08/03/2024 - Jeferson E.R Arajo

Manutenes:
__/__/____ - _______ ___________
__/__/____ - _______ ___________
__/__/____ - _______ ___________
__/__/____ - _______ ___________
*/
create or replace FUNCTION POSESTCONS_LOTE_VALIDO(
   P_FILIAL        IN POSESTDIA.CDFILIAL%TYPE,
   P_PRODUTO       IN POSESTDIA.CDPRODUTO%TYPE,
   P_ALMOXARIFE    IN POSESTDIA.CDALMOXARIFE%TYPE,
   P_CDLOCALESTOQ  IN POSESTDIA.CDLOCALESTOQ%TYPE,
   P_IDPRODACABADO IN VARCHAR2 DEFAULT 'N',
   P_DTPOSICAO     IN POSESTDIA.DTPOSIESTQ%TYPE DEFAULT SYSDATE,
   P_DTREFVALIDADE IN POSESTDIA.DTPOSIESTQ%TYPE DEFAULT SYSDATE 
   ) RETURN FLOAT
AS
   V_IDUTILALMOX  VARCHAR2(01);
   V_IDUTILLCESTQ VARCHAR2(01);
   V_ALMOXARIFE   VARCHAR2(06);
   V_LOCALESTOQ   VARCHAR2(06);
   V_NRDIASLIMSEGVAL NUMBER;
   V_DTPOSICAO DATE;
   V_DTREFVALIDADE DATE;
   V_QTDEESTO FLOAT;

BEGIN

    BEGIN
        SELECT NVL(MAX(IDUTILALMOX),'N'), NVL(MAX(IDUTILLCESTQ),'N')
          INTO V_IDUTILALMOX, V_IDUTILLCESTQ
          FROM PARAMFILIAL
          WHERE CDFILIAL = P_FILIAL;
    EXCEPTION
    WHEN NO_DATA_FOUND THEN
        V_IDUTILALMOX  :='N';
        V_IDUTILLCESTQ :='N';
    END;

    BEGIN
        SELECT NVL(NRDIASLIMSEGVAL,0)
         INTO V_NRDIASLIMSEGVAL /*Campo Limite Seg. Validade(Dias) - Tela Produtos com Controle de Estoque*/
         FROM PRODFILI 
    WHERE CDFILIAL  = P_FILIAL AND 
          CDPRODUTO = P_PRODUTO;
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
        V_NRDIASLIMSEGVAL :=0;
    END;

    V_DTPOSICAO     := (P_DTPOSICAO+1);
    
    /*Se o P_ALMOXARIFE foi informado  o almoxarifado que ser consultado, caso contrario
    a rotina em ELSE e detrmina qual almoxarifado deve ser consultado com base em parametrizaes*/
    IF P_ALMOXARIFE IS NOT NULL THEN
        V_ALMOXARIFE := P_ALMOXARIFE;
    ELSE
        /***************************************************************************************************************************
        1 - Se a unidade controla ALMOXARIFADO consulta os dados em "Local de Estoque por Produto". 
            Caso o ALMOXARIFADO esteja nulo, consulta o ALMOXARIFADO padro da tela "Parametros de Estoque" aba "Filial".
        2 - Se a unidade no controla ALMOXARIFADO o valor ' ' ser atribuido a respectiva variavel.
    
        3 - Se o parametro P_IDPRODACABADO  igual a S indica que o ALMOXARIFADO consultado em "Local de Estoque Por Produto" 
             o de "Produto acabado", caso contrario ir consultar o ALMOXARIFADO de "Materia-Prima/Insumos para Produo".
        ****************************************************************************************************************************/  
        BEGIN
            SELECT DECODE(V_IDUTILALMOX,'S', NVL(DECODE(P_IDPRODACABADO,'S',PF.CDALMOXPRACA,PF.CDALMOXINSPR),NVL(P.CDALMOXARIFE,' ')),' ')
              INTO V_ALMOXARIFE
              FROM PRODFILI PF, PARAMFILIAL P
             WHERE PF.CDFILIAL  = P.CDFILIAL 
               AND PF.CDFILIAL  = P_FILIAL
               AND PF.CDPRODUTO = P_PRODUTO;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                V_ALMOXARIFE := ' ';   
        END;
    END IF;
    
    
    /*Se o P_CDLOCALESTOQ foi informado  o Local de Estoque que ser consultado, caso contrario
    a rotina em ELSE e detrmina qual local de estoque deve ser consultado com base em parametrizaes*/
    IF P_CDLOCALESTOQ IS NOT NULL THEN
        V_LOCALESTOQ := P_CDLOCALESTOQ;
    ELSE
        /***************************************************************************************************************************
        1 - Se a unidade controla LOCAL DE ESTOQUE consulta os dados em "Local de Estoque por Produto". 
            Caso o LOCAL DE ESTOQUE esteja nulo, consulta o LOCAL DE ESTOQUE padro da tela "Parametros de Estoque" aba "Filial".
        2 - Se a unidade no controla LOCAL DE ESTOQUE o valor ' ' ser atribuido a respectiva variavel.
    
        3 - Se o parametro P_IDPRODACABADO  igual a S indica que o LOCAL DE ESTOQUE consultado em "Local de Estoque Por Produto" 
             o de "Produto acabado", caso contrario ir consultar o almoxarifado de "Materia-Prima/Insumos para Produo".
        ****************************************************************************************************************************/  
        BEGIN
            SELECT DECODE(V_IDUTILLCESTQ,'S',NVL(  DECODE(P_IDPRODACABADO,'S',PF.CDLOCALPRACA,PF.CDLOCALINSPR) ,NVL(P.CDLOCALESTOQ,' ')),' ')
              INTO V_LOCALESTOQ
              FROM PRODFILI PF, PARAMFILIAL P
             WHERE PF.CDFILIAL  = P.CDFILIAL 
               AND PF.CDFILIAL  = P_FILIAL
               AND PF.CDPRODUTO = P_PRODUTO;
        EXCEPTION
            WHEN NO_DATA_FOUND THEN
                V_LOCALESTOQ := ' ';    
        END;
    END IF;

    BEGIN
        SELECT  NVL(SUM(QTESTOQDIA),0) AS QTDEESTO
          INTO V_QTDEESTO
          FROM POSESTDIA P, LOTE L 
          WHERE (P.CDFILIAL     = P_FILIAL)       AND 
                (P.CDALMOXARIFE = V_ALMOXARIFE)   AND 
                (P.CDLOCALESTOQ = V_LOCALESTOQ)   AND 
                (P.CDPRODUTO    = P_PRODUTO   )   AND
                (P.CDFILIAL     = L.CDFILIAL(+))   AND
                (P.CDPRODUTO    = L.CDPRODUTO(+))  AND
                (P.NRLOTEESTQ   = L.NRLOTEESTQ(+)) AND
                (P.NRSUBLOTE    = L.NRSUBLOTE(+))  AND
                /*A data de validade do lote(quando usado) precisa ser maior ou igual a data de referencia(que considera dias de limite de segurana)*/
                ((NVL(L.DTVALILOTE,SYSDATE) - DECODE(L.DTVALILOTE,NULL,0,V_NRDIASLIMSEGVAL))>= TRUNC(P_DTREFVALIDADE)) AND 
                (P.DTPOSIESTQ  = (SELECT DTPOSIESTQ
                                    FROM (SELECT A.DTPOSIESTQ
                                            FROM POSESTDIA A
                                           WHERE A.CDFILIAL     = P.CDFILIAL
                                             AND A.CDALMOXARIFE = P.CDALMOXARIFE
                                             AND A.CDLOCALESTOQ = P.CDLOCALESTOQ
                                             AND A.CDPRODUTO    = P.CDPRODUTO
                                             AND A.NRLOTEESTQ   = P.NRLOTEESTQ
                                             AND A.NRSUBLOTE    = P.NRSUBLOTE
                                             AND A.DTPOSIESTQ   <= V_DTPOSICAO
                                          ORDER BY 1 DESC)
                                  WHERE ROWNUM = 1));
    EXCEPTION
        WHEN NO_DATA_FOUND THEN
        V_QTDEESTO := 0;
    END;
    RETURN(V_QTDEESTO);
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('POSESTCONS_LOTE_VALIDO','6.49.044'); END;
/

